-- STM : Simple Threat Meter
-- 2009-2010 Amethyst
--

SimpleThreatMeter = LibStub("AceAddon-3.0"):NewAddon("SimpleThreatMeter", "AceConsole-3.0", "AceEvent-3.0");
local mod = SimpleThreatMeter;
local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");
local LSM = LibStub("LibSharedMedia-3.0");
local db;

local inRaid, inParty, inCombat;	-- boolean variables indicating if the player is in a raid and/or party.
local lastWarning = 0;				-- How long ago the last warning was, to stop prevent warning spam.
local mirrorImageValue = 410065408;	-- The threat reduction from mirror image / fade. NOT USED.
local currentPlayerSpec = 0;		-- The player talent spec. 1=main, 2=alternate.
local animationSmoothness = 10;		-- The larger the value, the smoother (but slower) bar animation will be.
local colorPhaseRed = {0,0,0,0,0,0,0,0,0,0,0.2,0.4,0.6,0.8,1,1,1,1,1,1};
local colorPhaseGreen = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0.8,0.6,0.4,0.2,0};

-- These are some of the status bars from SharedMedia, but rotated through 90 degrees for vertical bars.
-- Probably the best way to handle this for the future is to seek a change to the shared media distribution?
LSM:Register("statusbar_v", "Charcoal",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Charcoal]]);
LSM:Register("statusbar_v", "Glaze2",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Glaze2]]);
LSM:Register("statusbar_v", "Healbot",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Healbot]]);
LSM:Register("statusbar_v", "Lyfe",				[[Interface\Addons\SimpleThreatMeter\statusbarv\Lyfe]]);
LSM:Register("statusbar_v", "Perl2",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Perl2]]);
LSM:Register("statusbar_v", "Runes",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Runes]]);
LSM:Register("statusbar_v", "Smoothv2",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Smoothv2]]);
LSM:Register("statusbar_v", "Steel",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Steel]]);
LSM:Register("statusbar_v", "Wglass",			[[Interface\Addons\SimpleThreatMeter\statusbarv\Wglass]]);

if (LSM:Fetch("statusbar", "Healbot")) then
	-- we know that SharedMedia mod is loaded
else	-- register the standard bars ourselves if user doesn't have SharedMedia.
	LSM:Register("statusbar", "Charcoal",		[[Interface\Addons\SimpleThreatMeter\statusbar\Charcoal]]);
	LSM:Register("statusbar", "Glaze2",			[[Interface\Addons\SimpleThreatMeter\statusbar\Glaze2]]);
	LSM:Register("statusbar", "Healbot",		[[Interface\Addons\SimpleThreatMeter\statusbar\Healbot]]);
	LSM:Register("statusbar", "Lyfe",			[[Interface\Addons\SimpleThreatMeter\statusbar\Lyfe]]);
	LSM:Register("statusbar", "Perl2",			[[Interface\Addons\SimpleThreatMeter\statusbar\Perl2]]);
	LSM:Register("statusbar", "Runes",			[[Interface\Addons\SimpleThreatMeter\statusbar\Runes]]);
	LSM:Register("statusbar", "Smoothv2",		[[Interface\Addons\SimpleThreatMeter\statusbar\Smoothv2]]);
	LSM:Register("statusbar", "Steel",			[[Interface\Addons\SimpleThreatMeter\statusbar\Steel]]);
	LSM:Register("statusbar", "Wglass",			[[Interface\Addons\SimpleThreatMeter\statusbar\Wglass]]);
end

local texture_array = {
		["Charcoal"]	= "Charcoal",
		["Glaze2"]		= "Glaze2",
		["Healbot"]		= "Healbot",
		["Lyfe"]		= "Lyfe",
		["Perl2"]		= "Perl2",
		["Runes"]		= "Runes",
		["Smoothv2"]	= "Smoothv2",
		["Steel"]		= "Steel",
		["Wglass"]		= "Wglass",
	}


--
-- OPTIONS SECTION
--

-- Next two functions may not be needed?
local function getOption(info)
	return (info.arg and mod.db.profile[info.arg] or mod.db.profile[info[#info]]);
end;

local function setOption(info, value)
	local key = info.arg or info[#info];
	mod.db.profile[key] = value;
end;

mod.version = GetAddOnMetadata("SimpleThreatMeter", "Version");

local options = {
	name = "SimpleThreatMeter",
	handler = SimpleThreatMeter,
	type = "group",
	args = {
		maindesc = {
			type = 'description',
			name = "|cff40ff40Simple Threat Meter|cffffffff version |cff40ff40" .. mod.version .. "|cffffffff by |cff9966ffAmethyst|cffffffff. |r|n|nSimple Threat Meter uses the built in Blizzard threat logic, which differentiates between melee & ranged automatically, thus presenting a bar that means YOU PULL AGGRO if YOU HIT 100%!|r|nSimple!|r|n|cffff4040Note: Simple Threat Meter WILL display a different percent value to Omen. This is not a bug.|r|n",
			order = 5,
		},
		enabled = {
			type = 'toggle',
			name = "Enabled",
			desc = "Enable Simple Threat Meter mod",
			get = function() return mod.db.profile.enabled end,
			set = function(info, n) mod.db.profile.enabled = n mod:ShowWhen() end,
			order = 10,
		},
		locked = {
			type = 'toggle',
			name = "Locked",
			desc = "Lock the mod so it cannot be moved. Whilst locked, you can click through the threat meter to target objects behind it.",
			get = function() return mod.db.profile.locked end,
			set = function(info, n)
				mod.db.profile.locked=n;
				mod:Resized();
			end,
			order = 15,
		},
		threatbartest = {
			order = 20,
			type = 'execute',
			name = "Test",
			desc = "Display a test amount of threat on the threat bar.",
			func = function()
					mod:setNewThreatValue(85);
					mod:Resized();
				end,
		},
	},
}

local options_size = {
	name = "Simple Threat Meter - Size & Appearance",
	type = "group",
	args = {
		basicbaroptions = {
			type = "group",
			order = 10,
			name = "|cff40ff40Basic Layout Options|r",
			inline = true,
			args = {
				vertical = {
					order = 10,
					type = 'toggle',
					name = "Vertical layout",
					width = "full",
					desc = "The orientation of the bar. Switch between Vertical and Horizontal layout.",
					get = function() return mod.db.profile.vertical end,
					set = function(info, n)
						mod.db.profile.vertical=n;
						mod:Resized();
					end,
				},
				thickness = {
					order = 20,
					type = "range",
					name = "Bar Thickness",
					desc = "This is how thick the bar is. If your bar is Vertical, this will be the width. If your bar is Horizontal this is it's height.",
					min = 10,
					max = 100,
					step = 5,
					width = "double",
					get = function(info)
						return mod.db.profile.thickness
					end,
					set = function(info, n)
						mod.db.profile.thickness = n;
						mod:setNewThreatValue(75);
						mod:Resized();
					end,
				},
				barlength = {
					order = 30,
					type = "range",
					name = "Bar Length",
					desc = "This is how long the bar is. If your bar is Vertical, this will be the height. If your bar is Horizontal this is it's width.",
					min = 120,
					max = 800,
					step = 10,
					width = "double",
					get = function(info)
						return mod.db.profile.length
					end,
					set = function(info, n)
						mod.db.profile.length = n;
						mod:setNewThreatValue(75);
						mod:Resized();
					end,
				},
			},
		},
		threatbaroptions = {
			type = "group",
			order = 20,
			name = "|cff40ff40Threat Bar Options|r",
			inline = true,
			args = {
				backgroundcolor ={
					order = 10,
					type = "color",
					hasAlpha = true,
					name = "Background color",
					desc = "The colour and transparency of the background",
					get = function(info)
						local c = mod.db.profile.Background.Color;
						return c.r, c.g, c.b, c.a;
					end,
					set = function(info, r, g, b, a)
						local c = mod.db.profile.Background.Color;
						c.r, c.g, c.b, c.a = r, g, b, a;
						mod:setNewThreatValue(75);
						mod:Resized();
					end,					
				},
				bordercolor ={
					order = 20,
					type = "color",
					hasAlpha = true,
					name = "Border color",
					desc = "The colour and transparency of the border.",
					get = function(info)
						local c = mod.db.profile.Border.Color;
						return c.r, c.g, c.b, c.a;
					end,
					set = function(info, r, g, b, a)
						local c = mod.db.profile.Border.Color;
						c.r, c.g, c.b, c.a = r, g, b, a;
						mod:setNewThreatValue(75);
						mod:Resized();
					end,					
				},
				foregroundcolor ={
					order = 30,
					type = "color",
					hasAlpha = true,
					name = "Threat bar color",
					desc = "The colour and transparency of the threat bar",
					get = function(info)
						local c = mod.db.profile.Foreground.Color;
						return c.r, c.g, c.b, c.a;
					end,
					set = function(info, r, g, b, a)
						local c = mod.db.profile.Foreground.Color;
						c.r, c.g, c.b, c.a = r, g, b, a;
						mod:setNewThreatValue(75);
						mod:Resized();
					end,					
				},
				gradualColour = {
					order = 35,
					type = 'toggle',
					name = "Colour based on Threat?",
--					width = "full",
					desc = "If active, the colour of the threat bar will gradual change from green to red as you get higher in threat.",
					get = function() return mod.db.profile.gradualColour end,
					set = function(info, n)
						mod.db.profile.gradualColour=n;
						mod:Resized();
					end,
				},
				threattexture = {
					order = 40,
	    			type = "select",
	    			style = "dropdown",
	    			name = "Threat bar texture",
--	    			width = "double",
	    			desc = "the texture of your threat bar",
	    			values = function ()
							return texture_array
	    				end,
					get = function(info)
						return mod.db.profile.Foreground.Texture;
					end,
					set = function(info, n)
						mod.db.profile.Foreground.Texture = n;
						mod:Print("Selected texture: " .. mod.db.profile.Foreground.Texture);
						mod:setNewThreatValue(75);
						mod:Resized();
					end,
				},
			},
		}, -- threatbaroptions
	},
}

local options_text = {
	name = "Simple Threat Meter - Text Options",
	type = "group",
	args = {
		threatfontname = {
			order = 10,
			type = "select",
			style = "dropdown",
			name = "Font",
			desc = "The font to use to display threat levels / name.",
			values = function ()
					local return_array = {};
					local fontArray = LSM:List('font');
					for i, fname in pairs(fontArray) do
						return_array[fname] = fname;
					end
					return return_array	    					
				end,
			get = function(info)
				return mod.db.profile.threatfontname
			end,
			set = function(info, n)
				mod.db.profile.threatfontname = n;
				mod.threatText:SetText("95%");
				mod:Resized();
			end,
		},
		threattextlocation = {
			order = 20,
			type = "select",
			style = "dropdown",
			name = "Text location",
			desc = "Where to display the text in location to the bar.|n|cffff6000This is only relevant to Horizontal mode!|r",
			values = function ()
					return_array = {
						[0] = "Outside Left",
						[1] = "Underneath",
						[2] = "Above",
						[3] = "Inside bar",
					}
					return return_array	    					
				end,
			get = function(info)
				return mod.db.profile.threattextlocation
			end,
			set = function(info, n)
				mod.db.profile.threattextlocation = n;
				mod.threatText:SetText("95%");
				mod:setNewThreatValue(75);
				mod:Resized();
			end,
		},
		threattextoptions = {
			type = "group",
			order = 30,
			name = "|cff40ff40Threat Text Options|r",
			inline = true,
			args = {
				threatfontsize = {
					order = 10,
					type = "range",
					name = "Threat Font Size",
					desc = "The size of font to display the current percentage threat.",
					min = 8,
					max = 32,
					step = 1,
					get = function(info)
						return mod.db.profile.threatfontsize
					end,
					set = function(info, n)
						mod.db.profile.threatfontsize = n;
						mod.threatText:SetText("95%");
						mod:Resized();
					end,
				},
				threattextcolor ={
					order = 20,
					type = "color",
					hasAlpha = true,
					name = "Threat text color",
					desc = "The colour and transparency of the text that displays your threat level",
					get = function(info)
						local c = mod.db.profile.ThreatText.Color;
						return c.r, c.g, c.b, c.a;
					end,
					set = function(info, r, g, b, a)
						local c = mod.db.profile.ThreatText.Color;
						c.r, c.g, c.b, c.a = r, g, b, a;
						mod:Resized();
					end,					
				},
				threatdisplaytype = {
					order = 40,
	    			type = "select",
	    			style = "dropdown",
	    			name = "Threat text displayed",
	    			width = "double",
	    			desc = "Format to display your threat level.",
	    			values = function ()
							return_array = {
								[0] = "None",
								[1] = "Percentage threat",
								[2] = "Absolute threat",
--								[3] = "Percentage & Absolute",
							}
							return return_array	    					
	    				end,
					get = function(info)
						return mod.db.profile.threatdisplaytype
					end,
					set = function(info, n)
						mod.db.profile.threatdisplaytype = n;
					end,
				},
			},
		}, -- threattextoptions
		threattargetoptions = {
			type = "group",
			order = 50,
			name = "|cff40ff40Target Name Display Options|r",
			inline = true,
			args = {
				threattargetfontsize = {
					order = 10,
					type = "range",
					name = "Target Font Size",
					desc = "The size of font to display the current target.",
					min = 8,
					max = 32,
					step = 1,
					get = function(info)
						return mod.db.profile.threattargetfontsize
					end,
					set = function(info, n)
						mod.db.profile.threattargetfontsize = n;
						mod:Resized();
					end,
				},
				showtargetname = {
					order = 20,
					type = 'toggle',
					name = "Show target name",
					desc = "Display the name of your current target.",
					get = function() return mod.db.profile.showtargetname end,
					set = function(info, n)
						mod.db.profile.showtargetname=n;
						mod:PLAYER_TARGET_CHANGED();
					end,
				},
			},
		}, -- threattargetoptions
	},
}

local options_conditions = {
	name = "Simple Threat Meter - Conditions",
	type = "group",
	args = {
		visibleoptions1 = {
			type = "group",
			order = 10,
			name = "|cff40ff40Show when|r",
			inline = true,
			args = {
				solo = {
					type = 'toggle',
					name = "You are solo",
					desc = "Show STM when you are alone.",
					get = function() return mod.db.profile.ShowWhen.Alone end,
					set = function(info, n)
						mod.db.profile.ShowWhen.Alone=n;
						mod:ShowWhen();
					end,
					order = 5,
				},
				party = {
					type = 'toggle',
					name = "You are in a Group",
					desc = "Show STM when you are in a group.",
					get = function() return mod.db.profile.ShowWhen.Party end,
					set = function(info, n)
						mod.db.profile.ShowWhen.Party=n;
						mod:ShowWhen();
					end,
					order = 10,
				},
				raid = {
					type = 'toggle',
					name = "You are in a Raid",
					desc = "Show STM when you are in a raid.",
					get = function() return mod.db.profile.ShowWhen.Raid end,
					set = function(info, n)
						mod.db.profile.ShowWhen.Raid=n;
						mod:ShowWhen();
					end,
					order = 15,
				},
				pet = {
					type = 'toggle',
					name = "You have a Pet",
					desc = "Always show STM if you have a pet summoned.",
					get = function() return mod.db.profile.ShowWhen.Pet end,
					set = function(info, n)
						mod.db.profile.ShowWhen.Pet=n;
						mod:ShowWhen();
					end,
					order = 20,
				},
			},
		},
		visibleoptions2 = {
			type = "group",
			order = 20,
			name = "|cff40ff40Hide When|r",
			inline = true,
			args = {
				break03 = {
					type = "description",
					name = "|cff00ddffIn addition, the following three options will override the four settings above, when they are true.|r",
					order = 21,
				},
				HideWhenOOC = {
					type = 'toggle',
					name = "Hide out of combat",
					desc = "Hide STM when you are not in combat.",
					get = function() return mod.db.profile.ShowWhen.HideWhenOOC; end,
					set = function(info, n)
						mod.db.profile.ShowWhen.HideWhenOOC=n;
						mod:ShowWhen();
					end,
					order = 25,
				},
				HideWhenPVP = {
					type = 'toggle',
					name = "Hide in PvP",
					desc = "Hide STM when you are in ARENA or a BATTLEGROUND.",
					get = function() return mod.db.profile.ShowWhen.HideInPVP; end,
					set = function(info, n)
						mod.db.profile.ShowWhen.HideInPVP=n;
						mod:ShowWhen();
					end,
					order = 30,
				},
				HideWhenNoTarget = {
					type = 'toggle',
					name = "Hide when no target",
					desc = "Hide STM when you have no target.",
					get = function() return mod.db.profile.ShowWhen.HideWhenNoTarget; end,
					set = function(info, n)
						mod.db.profile.ShowWhen.HideWhenNoTarget=n;
						mod:ShowWhen();
					end,
					order = 35,
				},
				break04 = {
					type = "description",
					name = "|cff00ddffYou may automate disabling STM for specific talent builds, such as when you are tanking.|r",
					order = 40,
				},
				HideWhenSpecMain = {
					type = 'toggle',
					name = "Hide for Main Spec",
					desc = "Hide STM when your PRIMARY talent build is active.",
					get = function() return mod.db.profile.ShowWhen.HideWhenTalentsMain; end,
					set = function(info, n)
						mod.db.profile.ShowWhen.HideWhenTalentsMain=n;
						mod:ShowWhen();
					end,
					order = 50,
				},
				HideWhenSpecSecondary = {
					type = 'toggle',
					name = "Hide for Off Spec",
					desc = "Hide STM when your SECONDARY talent build is active.",
					get = function() return mod.db.profile.ShowWhen.HideWhenTalentsSecondary; end,
					set = function(info, n)
						mod.db.profile.ShowWhen.HideWhenTalentsSecondary=n;
						mod:ShowWhen();
					end,
					order = 55,
				},
			},
		}, -- visibleoptions
	},
}

local options_warnings = {
	name = "Simple Threat Meter - Warnings",
	type = "group",
	args = {
		threatwarningtext = {
			type = "description",
			name = "|cff00ddffSTM can warn you if you are about to pull aggro, visually or by playing a sound effect.|r",
			width = "full",
			order = 5,
		},
		threatwarningthreshold = {
			order = 10,
			type = "range",
			name = "Warning Threshold",
			desc = "Give a warning once threat reaches this % value.",
			min = 50,
			max = 100,
			step = 1,
			get = function(info)
				return mod.db.profile.warning.Level;
			end,
			set = function(info, n)
				mod.db.profile.warning.Level = n;
				mod:Resized();
			end,
		},
		threatwarningcooldown = {
			order = 20,
			type = "range",
			name = "Warning Cooldown",
			desc = "Prevents spamming of warning. Leave this many seconds between warnings.",
			min = 1,
			max = 10,
			step = 1,
			get = function(info)
				return mod.db.profile.warning.Cooldown;
			end,
			set = function(info, n)
				mod.db.profile.warning.Cooldown = n;
				mod:Resized();
			end,
		},
		threatwarning = {
			type = "group",
			order = 30,
			name = "|cff40ff40Audio Warnings|r",
			inline = true,
			args = {
--				threatwarningvisual = {
--					order = 20,
--					type = 'toggle',
--					name = "Give a visual warning?",
--					desc = "Enable whether to display a text threat warning at the specified threat level.",
--					get = function() return mod.db.profile.warning.Visual end,
--					set = function(info, n)
--						mod.db.profile.warning.Visual=n;
--					end,
--				},
				threatwarningaudio = {
					order = 10,
					type = 'toggle',
					name = "Give an audio warning?",
					desc = "Enable whether to sound an audible threat warning at the specified threat level.",
					get = function() return mod.db.profile.warning.Audio end,
					set = function(info, n)
						mod.db.profile.warning.Audio=n;
					end,
				},
				break05 = {
					type = "description",
					name = "",
					order = 20,
				},
				threatwarningaudiosound = {
					order = 30,
	    			type = "select",
	    			style = "dropdown",
	    			name = "Sound effect",
--	    			width = "double",
	    			desc = "The sound to play for a threat warning.",
	    			values = function ()
							return_array = {
								["aggroalert.ogg"] = "Aggro Alert",
--								["alarm2.ogg"] = "Alarm",
--								["arrow-hit.ogg"] = "Arrow",
--								["attack.ogg"] = "Attack",
--								["attention.ogg"] = "Attention",
								["bunny.ogg"] = "Bunny",
								["doh.ogg"] = "Doh!",
--								["dramatic.ogg"] = "Dramatic",
--								["multi.ogg"] = "Multi sound",
								["referee.ogg"] = "Ref whistle",
--								["slap.ogg"] = "Slap",
--								["throwknife.ogg"] = "Thrown knife",
							}
							return return_array	    					
	    				end,
					get = function(info)
						return mod.db.profile.warning.threatwarningaudiosound
					end,
					set = function(info, n)
						mod.db.profile.warning.threatwarningaudiosound = n;
					end,
				},
				threatwarningaudiotest = {
					order = 40,
					type = 'execute',
					name = "Test",
					desc = "Test the currently selected audio warning.",
					func = function()
							local Soundfile = "Interface\\AddOns\\SimpleThreatMeter\\sounds\\" .. mod.db.profile.warning.threatwarningaudiosound;
							PlaySoundFile(Soundfile);
						end,
				},
			},
		}, -- threatwarning
	},
}

local profile_options = {
	name = "SimpleThreatMeter Profiles",
	type = "group",
	args = {
	},
}

-- Default
local defaults = {
	profile = {
		enabled = true,
		locked = false,
		thickness = 20,
		length = 200,
		vertical = false,
		threatfontsize = 16,
		threattargetfontsize = 16,
		threatfontname = "Friz Quadrata TT",
		threattextlocation = 0,
		showtargetname = true,
		threatdisplaytype = 1,
		screenx = 600,
		screeny = 500,
		gradualColour = true,
		Background = {
			Color = {r = 0, g = 0, b = 0, a = 0.8,},
		},
		Foreground = {
			Texture = "Smoothv2",
			Color = {r = 0, g = 1, b = 0, a = 0.8},
		},
		ThreatText = {
			Color = {r = 1, g = 1, b = 1, a = 1},
		},
		Border = {
			Color = {r = 0, g = 0, b = 0, a = 0.6},
		},
		ShowWhen = {
			Alone = true,
			Pet = true,
			Party = true,
			Raid = true,
			HideWhileResting = true,
			HideInPVP = true,
			HideWhenOOC = false,
			HideWhenNoTarget = false,
			HideWhenTalentsMain = false,
			HideWhenTalentsSecondary = false,
		},
		warning = {
			Audio = true,
			Visual = true,
			Level = 85,
			Cooldown = 4,
			threatwarningaudiosound = "aggroalert.ogg",
		},
	},
}

--
-- CORE FUNCTIONS
--

-- OpenConfig
function mod:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
end

function mod:OnInitialize()
    -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("SimpleThreatMeterDB", defaults, "Default");

	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleThreatMeter", options);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Simple Threat Meter - Size & Appearance", options_size);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Simple Threat Meter - Text Options", options_text);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Simple Threat Meter - Warnings", options_warnings);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Simple Threat Meter - Conditions", options_conditions);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleThreatMeter Profiles", profile_options);
	profile_options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleThreatMeter", "SimpleThreatMeter");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Simple Threat Meter - Size & Appearance", "Size & Appearance", "SimpleThreatMeter");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Simple Threat Meter - Text Options", "Text Options", "SimpleThreatMeter");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Simple Threat Meter - Conditions", "Conditions", "SimpleThreatMeter");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Simple Threat Meter - Warnings", "Warnings", "SimpleThreatMeter");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleThreatMeter Profiles", "Profiles", "SimpleThreatMeter");

	self:RegisterChatCommand("simplethreatmeter", "ChatCommand");
	self:RegisterChatCommand("stm", "ChatCommand");
	
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

end

function mod:OnEnable()
    -- Called when the addon is enabled
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("UNIT_THREAT_LIST_UPDATE");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("UNIT_PET", "PARTY_MEMBERS_CHANGED"); -- check if they summon or dismiss pet.
	self:RegisterEvent("PLAYER_PET_CHANGED", "PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	currentPlayerSpec = GetActiveTalentGroup(false,false);
	self:CreateCoreFrames();
end

function mod:OnDisable()
    -- Called when the addon is disabled
end

function mod:OnProfileChanged(info)
	self:ShowWhen();
	self:Resized();
end

function mod:PLAYER_REGEN_ENABLED()
	-- Player is now out of combat.
	inCombat = false;
	self:UpdateThreatValue();
	self:ShowWhen();
end

function mod:PLAYER_REGEN_DISABLED()
	-- Player has entered combat.
	inCombat = true;
	self:ShowWhen();
end

function mod:UNIT_THREAT_LIST_UPDATE(unitID)
	-- Threat list has changed on a unit in range.
	self:UpdateThreatValue();
end

function mod:PLAYER_TARGET_CHANGED()
	-- Player has changed to looking at a different target
	if UnitExists("target") then
		if (mod.db.profile.showtargetname) then
			self.threatTargetText:SetText(UnitName("target"));
			if UnitCanAttack("player", "target") then
				self.threatTargetText:SetTextColor(1, 0.25, 0.25);	-- RED for hostile
			else
				self.threatTargetText:SetTextColor(0.25, 1, 0.25);	-- GREEN for non-hostile
			end
		end
	else
		self.threatTargetText:SetText(" ");
	end
	mod:ShowWhen();
	self:UpdateThreatValue();
end

function mod:PARTY_MEMBERS_CHANGED()
	inParty = GetNumPartyMembers() > 0;
	inRaid = GetNumRaidMembers() > 0;
	self:ShowWhen();
end

function mod:ACTIVE_TALENT_GROUP_CHANGED()
	-- The player has just switched spec.
	currentPlayerSpec = GetActiveTalentGroup(false,false);
	self:ShowWhen();
end

-- Deals with initiating an OnUpdate loop to smoothly animate the bars to the correct level.
function mod:setNewThreatValue(threatVal)
	self.threatTarget = threatVal;	-- This is the target that the bar value should move towards.
	--self:Print("Setting animate target of " .. self.threatTarget);
	if (self.isAnimating) then
		-- don't take any action if the bar is already animating.
	else
		self.threatBar:SetScript("OnUpdate", function(self, elapsed)
			local curValue = mod.threatBar:GetValue();
			if (curValue == mod.threatTarget) then
				mod.threatBar:SetScript("OnUpdate", nil);
				--mod:Print("Stopping updates");
				mod.isAnimating = false;
			else
				local step = ceil( (abs(mod.threatTarget-curValue))/ animationSmoothness );
				if (curValue < mod.threatTarget) then
					curValue = curValue + step;
				else
					curValue = curValue - step;
				end
				mod.threatBar:SetValue(curValue);
				if (curValue > 0) then
					local sparkPos = (curValue * mod.db.profile.length / 100) -12;
					if (mod.db.profile.vertical) then
						mod.threatBar.Spark:SetPoint('BOTTOM', mod.threatBar, 'BOTTOM', 0, sparkPos);
					else
						mod.threatBar.Spark:SetPoint('LEFT', mod.threatBar, 'LEFT', sparkPos, 0);
					end
					mod.threatBar.Spark:Show();
				else
					mod.threatBar.Spark:Hide();
				end
			end
			-- If we're using graduated colours on the bar
			if (mod.db.profile.gradualColour) then
				local colorChart = ceil(curValue/5);
				mod.threatBar:SetStatusBarColor(colorPhaseRed[colorChart],colorPhaseGreen[colorChart],0, 1);
			end
		end)	-- end of the function.
	end
end

-- Calculate the current threat value and invoke display routines.
function mod:UpdateThreatValue()
	if (mod.db.profile.enabled) then	-- only bother checking and running this logic if mod is enabled.
		local threatUnit = "target"
		local newThreat = 0;
		local threatCalc = 0;
		local newThreatText = " ";
		if UnitExists(threatUnit) then
			if UnitCanAttack("player", threatUnit) then
				local _,threatstatus,threatpct,rawthreatpct,threatvalue = UnitDetailedThreatSituation("player", threatUnit);
				if (threatpct) then
					newThreat = ceil(threatpct);
					if (newThreat == 100) then
						newThreatText = "-- AGGRO --";
					elseif (mod.db.profile.threatdisplaytype == 1) then		-- Display % threat
						newThreatText = tostring(newThreat) .. "%";
					elseif (mod.db.profile.threatdisplaytype == 2) then		-- Display absolute threat
						if (threatvalue > 0) then
							local aggroThreshold = tonumber(threatvalue) / threatpct;
							threatCalc = ceil(aggroThreshold - (tonumber(threatvalue)/100));
							threatCalc = (ceil(threatCalc / 100))/10;
							newThreatText = "-" .. tostring(threatCalc) .. " k";
						else
							-- estimate threat of tank if player has no threat on the mob
							if (CheckInteractDistance("target",3)) then
								distanceMultiplier = 1.1;
							else
								distanceMultiplier = 1.3;
							end
							local _,_,_,_,tankthreatvalue = UnitDetailedThreatSituation("targettarget", threatUnit);
							if (tankthreatvalue) then
								threatCalc = (ceil(distanceMultiplier * tankthreatvalue / 10000))/10;
								if (threatCalc < 1) then
									threatCalc = 0;
								end
							else
								threatCalc = 0;
							end
							newThreatText = "-" .. tostring(threatCalc) .. " k";
						end
					elseif (mod.db.profile.threatdisplaytype == 3) then
	--					newThreatText = tostring(newThreat) .. "%";
					else
						newThreatText = " ";
					end
				end
			end
		end
		self:setNewThreatValue(newThreat);
		self.threatText:SetText(newThreatText);
		-- Check to see if we should give a warning. ONLY if threat bar is enabled.
		if (self.threatBar:IsVisible()) then
			if (newThreat >= mod.db.profile.warning.Level) then
				if ((lastWarning + mod.db.profile.warning.Cooldown) < time()) then
					-- do the warning
					lastWarning = time();
					if (mod.db.profile.warning.Audio) then
						local Soundfile = "Interface\\AddOns\\SimpleThreatMeter\\sounds\\" .. mod.db.profile.warning.threatwarningaudiosound;
						--self:Print(Soundfile);
						PlaySoundFile(Soundfile);
					end
					if (mod.db.profile.warning.Visual) then
					end
				else
					-- Don't do the warning, as we're within the anti-spam cooldown.
				end
			end
		end
	else
	end
end

function mod:CreateCoreFrames()
	self.threatBar=CreateFrame("STATUSBAR","SimpleThreatMeter",UIParent,"TextStatusBar");
	self.threatBar:SetOrientation("VERTICAL");
	self.threatBar:SetWidth(self.db.profile.thickness);
	self.threatBar:SetHeight(200);
	self.threatBar:ClearAllPoints();
	self.threatBar:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", self.db.profile.screenx, self.db.profile.screeny);
	self.threatBar:SetMinMaxValues(0,100);
	LSM:Fetch("statusbar_v", "Healbot")
	self.threatBar:SetStatusBarTexture("Solid");
	self.threatBar:SetStatusBarColor(1, 0, 0, 1);
	self.threatBar:SetValue(0);

	-- create a frame just to get the border around the bar
	local tBackground = LSM:Fetch("background", "Solid");
	self.threatBorderFrame = CreateFrame("Frame",nil,self.threatBar);
	self.threatBorderFrame:SetFrameStrata("BACKGROUND");
	self.threatBorderFrame:ClearAllPoints();
	self.threatBorderFrame:SetHeight(200);
	self.threatBorderFrame:SetWidth(200);
	self.threatBorderFrame:SetPoint("BOTTOMLEFT", self.threatBar, "BOTTOMLEFT", -5, -5);
	self.threatBorderFrame:SetBackdrop({
		bgFile = tBackground,
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile=1, tileSize = 16,
		edgeSize = 16,
		insets = {left=2, right=2, top=2, bottom=2},
	});
	self.threatBorderFrame:SetBackdropColor(0,0,0,0);

	-- Create a frame for the text to be displayed in.
	self.threatTextFrame=CreateFrame("FRAME",nil,self.threatBar);
	self.threatTextFrame:SetWidth(300);
	self.threatTextFrame:SetHeight(50);
	self.threatTextFrame:SetPoint("TOP",self.threatBar,"BOTTOM");
--	self.threatTextFrame:SetBackdrop({
--		bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
--		tile=1, tileSize=10, edgeSize=18, 
--		insets={left=2, right=2, top=2, bottom=2}
--	});
--	self.threatTextFrame:SetBackdropColor(0,1,0,0.5);
	self.threatText = self.threatTextFrame:CreateFontString(nil,"ARTWORK","GameFontNormal");
	self.threatText:SetPoint("BOTTOM",self.threatTextFrame,"BOTTOM");
	self.threatText:SetText(" ");
	self.threatTargetText = self.threatTextFrame:CreateFontString(nil,"ARTWORK","GameFontNormal");
	self.threatTargetText:SetPoint("TOP",self.threatTextFrame,"TOP");
	self.threatTargetText:SetText(" ");

	self.threatBar:SetScript("OnMouseDown",function()
		self.threatBar:StartMoving();
	end)
	self.threatBar:SetScript("OnMouseUp",function()
		self.threatBar:StopMovingOrSizing();
		self.db.profile.screenx = ceil(self.threatBar:GetLeft());
		self.db.profile.screeny = ceil(self.threatBar:GetBottom());
		--self:Print("Moved! " .. tostring(self.db.profile.screenx) .. " / " .. tostring(self.db.profile.screeny));
	end)

	self.threatBar.Spark = self.threatBar:CreateTexture(nil, 'OVERLAY');
	self.threatBar.Spark:SetTexture('Interface\\CastingBar\\UI-CastingBar-Spark');
	self.threatBar.Spark:SetPoint('LEFT', self.threatBar, 'LEFT', 0, 0);
	self.threatBar.Spark:SetHeight(50);
	self.threatBar.Spark:SetWidth(25);
	self.threatBar.Spark:SetBlendMode('ADD');
	self.threatBar.Spark:Hide();

	self:Resized();
	self:ShowWhen()
end

function mod:Resized()
	-- Set the font correctly.
	self.threatText:SetFont(LSM:Fetch("font", self.db.profile.threatfontname), self.db.profile.threatfontsize, "OUTLINE");
	self.threatTargetText:SetFont(LSM:Fetch("font", self.db.profile.threatfontname), self.db.profile.threattargetfontsize, "OUTLINE");

	-- Size and orientation of the statusbar
	if (mod.db.profile.vertical) then
		--self:Print("Resize: Vertical");
		self.threatBar:SetOrientation("VERTICAL");
		self.threatBar:SetWidth(self.db.profile.thickness);
		self.threatBar:SetHeight(self.db.profile.length);
		self.threatBar:ClearAllPoints();
		self.threatBar:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", self.db.profile.screenx, self.db.profile.screeny);
		self.threatTextFrame:ClearAllPoints();
		self.threatTextFrame:SetHeight(self.threatText:GetHeight() + self.threatTargetText:GetHeight() + 5);
		self.threatTextFrame:SetWidth(300);
		self.threatTextFrame:SetPoint("TOP",self.threatBar,"BOTTOM",0,-2);
		self.threatText:ClearAllPoints();
		self.threatText:SetPoint("BOTTOM",self.threatTextFrame,"BOTTOM");
		self.threatTargetText:ClearAllPoints();
		self.threatTargetText:SetPoint("TOP",self.threatTextFrame,"TOP");
		-- Colours & Textures
		self.threatBar:SetStatusBarTexture(LSM:Fetch("statusbar_v", mod.db.profile.Foreground.Texture));
		-- Background & Border
		--self.threatBorderFrame:ClearAllPoints();
		self.threatBorderFrame:SetWidth(self.db.profile.thickness + 10);
		self.threatBorderFrame:SetHeight(self.db.profile.length + 10);
		-- Spark
		self.threatBar.Spark:ClearAllPoints();
		self.threatBar.Spark:SetRotation(math.pi / 2);
		self.threatBar.Spark:SetHeight(25);
		self.threatBar.Spark:SetWidth(25 + mod.db.profile.thickness);
		self.threatBar.Spark:SetPoint('BOTTOM', self.threatBar, 'BOTTOM', 0, 0);
	else
		--self:Print("Resize: Horizontal");
		self.threatBar:SetOrientation("HORIZONTAL");
		self.threatBar:SetWidth(self.db.profile.length);
		self.threatBar:SetHeight(self.db.profile.thickness);
		-- Colours & Textures
		self.threatBar:SetStatusBarTexture(LSM:Fetch("statusbar", mod.db.profile.Foreground.Texture));
		-- Background & Border
		--self.threatBorderFrame:ClearAllPoints();
		self.threatBorderFrame:SetWidth(self.db.profile.length + 10);
		self.threatBorderFrame:SetHeight(self.db.profile.thickness + 10);
		-- Code to vary the position of the TEXT based upon threattextlocation
		self.threatTextFrame:ClearAllPoints();
		self.threatText:ClearAllPoints();
		self.threatTargetText:ClearAllPoints();
		if (mod.db.profile.threattextlocation == 0) then		-- Outside Left
			self.threatTextFrame:SetHeight(self.threatText:GetHeight() + self.threatTargetText:GetHeight() + 5);
			self.threatTextFrame:SetPoint("RIGHT",self.threatBar,"LEFT",-2,0);
			self.threatText:SetPoint("BOTTOMRIGHT",self.threatTextFrame,"BOTTOMRIGHT");
			self.threatTargetText:SetPoint("TOPRIGHT",self.threatTextFrame,"TOPRIGHT");
		elseif (mod.db.profile.threattextlocation == 1) then	-- Underneath
			self.threatTextFrame:SetHeight(max(self.threatText:GetHeight(),self.threatTargetText:GetHeight()));
			self.threatTextFrame:SetWidth(mod.db.profile.length);
			self.threatTextFrame:SetPoint("TOPLEFT",self.threatBar,"BOTTOMLEFT",0,-2);
			self.threatText:SetPoint("TOPRIGHT",self.threatTextFrame,"TOPRIGHT");
			self.threatTargetText:SetPoint("TOPLEFT",self.threatTextFrame,"TOPLEFT");
		elseif (mod.db.profile.threattextlocation == 2) then	-- Above
			self.threatTextFrame:SetHeight(max(self.threatText:GetHeight(),self.threatTargetText:GetHeight()));
			self.threatTextFrame:SetWidth(mod.db.profile.length);
			self.threatTextFrame:SetPoint("BOTTOMLEFT",self.threatBar,"TOPLEFT",0,4);
			self.threatText:SetPoint("BOTTOMRIGHT",self.threatTextFrame,"BOTTOMRIGHT");
			self.threatTargetText:SetPoint("BOTTOMLEFT",self.threatTextFrame,"BOTTOMLEFT");
		else													-- Inside Bar
			self.threatTextFrame:SetHeight(max(self.threatText:GetHeight(),self.threatTargetText:GetHeight()));
			self.threatTextFrame:SetWidth(mod.db.profile.length);
			self.threatTextFrame:SetPoint("LEFT",self.threatBar,"LEFT",0,0);
			self.threatText:SetPoint("LEFT",self.threatTextFrame,"LEFT",5,0);
			self.threatTargetText:SetPoint("LEFT",self.threatText,"RIGHT",10,0);
		end
		-- Spark
		self.threatBar.Spark:ClearAllPoints();
		self.threatBar.Spark:SetHeight(25 + mod.db.profile.thickness);
		self.threatBar.Spark:SetWidth(25);
		self.threatBar.Spark:SetPoint('LEFT', self.threatBar, 'LEFT', 0, 0);
	end

	if (mod.db.profile.locked) then
		self.threatBar:SetMovable(false);
		self.threatBar:EnableMouse(false);
	else
		self.threatBar:SetMovable(true);
		self.threatBar:EnableMouse(true);
	end

	local c = mod.db.profile.Background.Color;
--	self.threatBar:SetBackdropColor(c.r, c.g, c.b, c.a);
	self.threatBorderFrame:SetBackdropColor(c.r, c.g, c.b, c.a);
	c = mod.db.profile.Border.Color;
	self.threatBorderFrame:SetBackdropBorderColor(c.r, c.g, c.b, c.a);
	c = mod.db.profile.Foreground.Color;
	self.threatBar:SetStatusBarColor(c.r, c.g, c.b, c.a);
	c = mod.db.profile.ThreatText.Color;
	self.threatText:SetTextColor(c.r, c.g, c.b, c.a);
end

-- called when the enable/disable option set
function mod:ShowWhen()
	if (mod.db.profile.enabled) then
		if (mod.db.profile.ShowWhen.HideWhenTalentsMain and (currentPlayerSpec==1)) then
			self.threatBar:Hide();
			return;
		end
		if (mod.db.profile.ShowWhen.HideWhenTalentsSecondary and (currentPlayerSpec==2)) then
			self.threatBar:Hide();
			return;
		end
		if (inCombat) then
		else
			if (mod.db.profile.ShowWhen.HideWhenOOC) then
				self.threatBar:Hide();
				return;
			end
		end
		-- Hide if no target
		if (mod.db.profile.ShowWhen.HideWhenNoTarget) then
			if (UnitExists("target")) then
			else
				self.threatBar:Hide();
				return;
			end
		end
		if (inRaid) then -- PLAYER IS IN A RAID
			if (mod.db.profile.ShowWhen.Raid) then
				self.threatBar:Show();
			else
				self.threatBar:Hide();
			end
		elseif (inParty) then -- PLAYER IS IN A GROUP
			if (mod.db.profile.ShowWhen.Party) then
				self.threatBar:Show();
			else
				self.threatBar:Hide();
			end
		else -- PLAYER IS SOLO
			if (mod.db.profile.ShowWhen.Alone) then
				self.threatBar:Show();
			else
				-- check here for "ah yes, but if they have a pet show it anyway" options.
				if (UnitExists("pet") and mod.db.profile.ShowWhen.Pet) then
					self.threatBar:Show();
				else
					self.threatBar:Hide();
				end
			end
		end
		if (mod.db.profile.ShowWhen.HideInPVP) then
			local _,instanceType = IsInInstance();
			if ( (instanceType=="pvp") or (instanceType=="arena") ) then
				self.threatBar:Hide();
			end
		end
	else
		self.threatBar:Hide();
	end
end

function mod:Test()
	self:Print("Location (saved):  " .. tostring(self.db.profile.screenx) .. " / " .. tostring(self.db.profile.screeny));
	local actualX = ceil(self.threatBar:GetLeft());
	local actualY = ceil(self.threatBar:GetBottom());
	self:Print("Location (actual): " .. tostring(actualX) .. " / " .. tostring(actualY));
end

-- /script SimpleThreatMeter:Test()
-- /script SimpleThreatMeter.threatBar:SetPoint("BOTTOMLEFT", WorldFrame, "BOTTOMLEFT", x, y);
-- self.threatBar:SetPoint("BOTTOMLEFT", WorldFrame, "BOTTOMLEFT", self.db.profile.screenx, self.db.profile.screeny);

-- /script SimpleThreatMeter.threatBar:ClearAllPoints();
-- /script SimpleThreatMeter.threatBar:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 1194, 297);
-- /script PlaySoundFile("Interface\\AddOns\\SimpleThreatMeter\\sounds\\referee.wav");

-- /script SimpleThreatMeter:setNewThreatValue(75);
-- /script SimpleThreatMeter.threatText:SetText("75%");

-- /script SimpleThreatMeter.threatBar.Spark:SetPoint('LEFT', SimpleThreatMeter.threatBar, 'LEFT', 200, 0);